# -*- coding: utf-8 -*-
import logging
import re
from oncall_slackbot.integrations import nlp
from slackbot import bot, manager
from oncall_slackbot.slackclient import BlocksSlackClient
from oncall_slackbot.dispatcher import OnCallMessageDispatcher
from oncall_slackbot import settings

logger = logging.getLogger(__name__)


class OnCallBot(bot.Bot):
    """
    Override to instantiate custom versions of objects.
    """
    def __init__(self):
        # Do not call super init since we are going to override everything anyways
        self._client = BlocksSlackClient(
            settings.API_TOKEN,
            timeout=settings.TIMEOUT if hasattr(settings, 'TIMEOUT') else None,
            bot_icon=settings.BOT_ICON if hasattr(settings, 'BOT_ICON') else None,
            bot_emoji=settings.BOT_EMOJI if hasattr(settings, 'BOT_EMOJI') else None
        )
        self._plugins = manager.PluginsManager()
        # Load all plugins
        for plugin in settings.PLUGINS:
            self._plugins._load_plugins(plugin)
        self._dispatcher = OnCallMessageDispatcher(self._client, self._plugins, settings.ERRORS_TO)


def nlp_label_respond_to(match_label, flags=0):
    def wrapper(func):
        if nlp.is_backend_present():
            manager.PluginsManager.commands['nlp_label_respond_to'][re.compile(match_label, flags)] = func
            logger.info(f'registered nlp_label_respond_to plugin "{func.__name__}" to "{match_label}"')
        else:
            logger.warning(f'no NLP backend is configured to handle nlp_label_respond_to for "{func.__name__}"')
        return func

    return wrapper


def nlp_label_listen_to(match_label, flags=0):
    def wrapper(func):
        if nlp.is_backend_present():
            manager.PluginsManager.commands['nlp_label_listen_to'][re.compile(match_label, flags)] = func
            logger.info(f'registered nlp_label_listen_to plugin "{func.__name__}" to "{match_label}"')
        else:
            logger.warning(f'no NLP backend is configured to handle nlp_label_listen_to for "{func.__name__}"')
        return func

    return wrapper


if nlp.is_backend_present():
    if 'nlp_label_respond_to' not in manager.PluginsManager.commands:
        manager.PluginsManager.commands['nlp_label_respond_to'] = {}
    if 'nlp_label_listen_to' not in manager.PluginsManager.commands:
        manager.PluginsManager.commands['nlp_label_listen_to'] = {}




import logging
import re
from typing import Optional
from pygerduty.v2 import ContactMethod, User
from slackbot.bot import listen_to
from oncall_slackbot.dispatcher import OnCallMessage
from oncall_slackbot.integrations import pagerduty


LOGGER = logging.getLogger(__name__)
CONTACT_METHOD_EMOTICON_BY_TYPE = {
    'email_contact_method': ':email:',
    'phone_contact_method': ':slack_call:',
    # Do not show anything for push or SMS notifications, since they duplicate the phone contact
    'sms_contact_method': None,
    'push_notification_contact_method': None,
}


def _get_contact_method_message(contact_method: ContactMethod, add_label: bool) -> Optional[str]:
    emoticon = CONTACT_METHOD_EMOTICON_BY_TYPE.get(contact_method.type)
    if not emoticon or (hasattr(contact_method, 'enabled') and not contact_method.enabled):
        return None
    label = ''
    if add_label:
        label = f' ({contact_method.label})'
    return f'{emoticon} {contact_method.address}{label}'


@listen_to(r'who is (?:currently )?on(?:-| |)call', re.IGNORECASE)
def who_is_on_call(message: OnCallMessage):
    if not pagerduty.is_configured():
        LOGGER.debug('Pager duty settings are not configured, cannot retrieve current on call')
        return

    current_oncall = pagerduty.get_current_oncall()
    if not current_oncall:
        message.reply(f'No current on-call information found for schedule {pagerduty.get_schedule_id()}')
        return

    # Build up contact methods string
    contact_method_messages = []
    user_name = pagerduty.get_user_name(current_oncall.user)
    if user_name:
        contact_method_messages.append(f':slack: @{user_name}')
    contact_methods = current_oncall.user.contact_methods.list(time_zone='MDT')
    for contact_method in contact_methods:
        # Only add label if there is no user name
        contact_method_message = _get_contact_method_message(contact_method, not user_name)
        if contact_method_message:
            contact_method_messages.append(contact_method_message)

    # Get formatted end time
    end_time = pagerduty.get_humanized_datetime(current_oncall.end, current_oncall.user.time_zone)

    message.reply_webapi(f'{current_oncall.user.name} is currently on call', in_thread=True, blocks=[
        {
            "type": "section",
            "text": {
                "type": "mrkdwn",
                "text":
                    f'*<{current_oncall.user.html_url}|{current_oncall.user.name}>* is on call '
                    f'<{current_oncall.schedule.html_url}|until {end_time}>'
            }
        },
        {
            "type": "section",
            "text": {
                "type": "mrkdwn",
                "text": '\n'.join(contact_method_messages)
            }
        },
    ])



# -*- coding: utf-8 -*-

import logging

import slacker_blocks
from slackbot import slackclient

logger = logging.getLogger(__name__)


class BlocksSlackClient(slackclient.SlackClient):
    """
    Overrides to support blocks in calls.
    """

    def __init__(self, token, timeout=None, bot_icon=None, bot_emoji=None, connect=True):
        super(BlocksSlackClient, self).__init__(
            # Never connect since we'll do that here
            token, timeout=timeout, bot_icon=bot_icon, bot_emoji=bot_emoji, connect=False
        )
        # Replace the webapi with the blocks-supporting version
        if timeout is None:
            self.webapi = slacker_blocks.Slacker(self.token)
        else:
            self.webapi = slacker_blocks.Slacker(self.token, timeout=timeout)

        if connect:
            self.rtm_connect()

    def send_message(self, channel, message, attachments=None, blocks=None, as_user=True, thread_ts=None):
        self.webapi.chat.post_message(
                channel,
                message,
                username=self.login_data['self']['name'],
                icon_url=self.bot_icon,
                icon_emoji=self.bot_emoji,
                attachments=attachments,
                blocks=blocks,
                as_user=as_user,
                thread_ts=thread_ts)



# -*- coding: utf-8 -*-


'''
Pager duty configuration, these are optional but both must be specified in
order to enable pager duty integration.
'''
PAGERDUTY_TOKEN = None
PAGERDUTY_SCHEDULE_ID = None


'''
(Optional, only works if pager duty is configured) Configures the email domain used to determine user names from emails.
For example, emails of the form 'myuser@example.com' should configure this property with a value of 'example.com'
(exclude the @ prefix). This would result in 'myuser' being set as the user name. 
'''
PAGERDUTY_USERNAME_EMAIL_DOMAIN = None


'''
Spacy configuration, this is optional
'''
SPACY_MODEL = None


# Import all slackbot settings as well, this also handles loading env vars, etc
from slackbot.settings import *


if PLUGINS == ['slackbot.plugins']:
    # Override default plugins
    PLUGINS = [
        'oncall_slackbot.plugins',
        'slackbot.plugins',
    ]

